<?php
session_start();
require 'vendor/autoload.php';

require 'db.php'; 

$_SESSION['tableData'] = [];

if (!isset($_POST['values'])) {
    echo "<p>No data submitted for aggregation. Please go back and submit data.</p>";
    exit;
}

$values = $_POST['values'];

?>
<!DOCTYPE html>
<html lang="en">
<head>
    <!-- Google Tag Manager -->
    <script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start': new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0], j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src='https://www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);})(window,document,'script','dataLayer','GTM-5PJBBBV6');</script>
    <!-- End Google Tag Manager -->
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>FGL FP Sheet Data Overview</title>
    <link rel="shortcut icon" href="https://www.hirdaramani.com/themes/hirdaramani/assets/images/logo/favicon.png" />
    <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/5.1.3/css/bootstrap.min.css">
    <link rel="stylesheet" href="aggregateBom.css">
    
</head>
<body>
    <!-- Google Tag Manager (noscript) -->
<noscript><iframe src="https://www.googletagmanager.com/ns.html?id=GTM-5PJBBBV6"
height="0" width="0" style="display:none;visibility:hidden"></iframe></noscript>
<!-- End Google Tag Manager (noscript) -->
    <div class="container">
        <h1 class="text-center">FGL FP Sheet Data Overview</h1>
        <div class="text-center mb-4">
            <a href="downloadBom.php" class="btn btn-primary btn-lg">Download Excel</a>
        </div>
        <?php
        foreach ($values as $factory => $threshold) {
            $factoryEscaped = $conn->real_escape_string($factory);
            $thresholdFloat = floatval($threshold);
            $upperLimit = $thresholdFloat + ($thresholdFloat * 0.05);
            $cumulativeTotal = 0;
            $displayedRows = [];

        $query = "SELECT 
            t2.CUSTOMER_NAME AS Buyer, 
            t2.CONSIGNEE AS Buyer_Name, 
            t2.INVOICE_NUMBER AS F_Invoice, 
            t2.FACTORY, 
            t2.AX_FORMAT AS AX_Invoice,
            t2.SOB_DATE AS Date, 
            ABS(CAST(REPLACE(t2.FOB, ',', '') AS DECIMAL(10, 2))) AS FOB, 
            ABS(CAST(REPLACE(t2.PUR_VALUE, ',', '') AS DECIMAL(10, 2))) AS PUR_VALUE,
            CASE 
                WHEN t1.Invoice IS NOT NULL THEN 'Yes' 
                ELSE 'No' 
            END AS Availability
          FROM Table2 t2
          LEFT JOIN Table1 t1 ON t2.AX_FORMAT = t1.Invoice
          WHERE t2.FACTORY = '$factoryEscaped' 
            AND (t2.F__PMNT_STATUS IS NULL OR t2.F__PMNT_STATUS = '') 
            AND t2.ELIGIBILITY = 'TRUE' 
          ORDER BY STR_TO_DATE(t2.SOB_DATE, '%m/%d/%Y') ASC";

            
            $result = $conn->query($query);

            echo "<h2>Payment from direct receipts - $factory</h2>";
            if ($result && $result->num_rows > 0) {
                echo "<table class='table'><thead><tr><th>Buyer</th><th>Buyer Name</th><th>Final Invoice</th><th>AX Invoice</th><th>Factory Name</th><th>Date</th><th>Sales Value in US$</th><th>Purchase Value in US$</th><th>Profit</th><th>Profit%</th><th>AP Aging Availability</th></tr></thead><tbody>";
                while ($row = $result->fetch_assoc()) {
                    $totalPayable = $row['PUR_VALUE']; 
                    $fob = floatval(str_replace(',', '', $row['FOB']));
                    $profit = $fob - $totalPayable;
                    $profitPercentage = ($fob > 0) ? ($profit / $fob) * 100 : 0; //
                    if (($cumulativeTotal + $totalPayable) <= $upperLimit) {
                        $cumulativeTotal += $totalPayable;
                        echo "<tr><td>{$row['Buyer']}</td><td>{$row['Buyer_Name']}</td><td>{$row['F_Invoice']}</td><td>{$row['AX_Invoice']}</td><td>{$row['FACTORY']}</td><td>{$row['Date']}</td><td>{$row['FOB']}</td><td>{$row['PUR_VALUE']}</td><td>{$profit}</td><td>" . number_format($profitPercentage, 2) . "%</td><td>{$row['Availability']}</td></tr>";
                        $displayedRows[] = $row; 
                    }
                }
                echo "</tbody></table>";
                $_SESSION['tableData'][] = ['factory' => $factory, 'data' => $displayedRows]; 
            } else {
                echo "<p>No eligible data found for $factory.</p>";
            }
        }
        $conn->close();
        ?>
    </div>
</body>
</html>
