<?php
session_start();
require 'vendor/autoload.php';

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Style\NumberFormat;



if (!empty($_SESSION['tableData'])) {
    $spreadsheet = new Spreadsheet();
    $sheet = $spreadsheet->getActiveSheet();

    $currentRow = 1;

    foreach ($_SESSION['tableData'] as $table) {
        $title = "Payment from direct receipts - {$table['factory']}";
        $sheet->setCellValue('A' . $currentRow, $title);
        $sheet->getStyle('A' . $currentRow)->getFont()->setBold(true);
        $currentRow += 2;

        $headers = ['Buyer', 'Buyer Name', 'Final Invoice', 'AX Invoice', 'Factory Name', 'Date', 'Sales Value in US$', 'Purchase Value in US$', 'Profit', 'Profit%'];
        $column = 'A';
        foreach ($headers as $header) {
            $sheet->setCellValue($column++ . $currentRow, $header);
        }
        $sheet->getStyle('A' . $currentRow . ':' . $column . $currentRow)->getFont()->setBold(true);
        $currentRow++;

        foreach ($table['data'] as $rowData) {
            $column = 'A';

            // Handle null values
            $rowData = array_map(function ($value) {
                return $value ?? '';
            }, $rowData);

            // Define and calculate $totalPayable and $fob here
            $totalPayable = floatval(str_replace(',', '', $rowData['Total_payable'] ?? ''));
            $fob = floatval(str_replace(',', '', $rowData['FOB'] ?? ''));
            $profit = $fob - $totalPayable; // Calculate profit
            $profitPercentage = ($fob != 0) ? ($profit / $fob) * 100 : 0; // Calculate profit percentage, ensuring division by zero is handled

            $sheet->setCellValue($column++ . $currentRow, $rowData['Buyer'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Buyer_Name'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['F_Invoice'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Invoice'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Factory'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Date'] ?? '');
            // Format and set FOB, Total Payable, and Profit with comma separation
            $sheet->setCellValue($column . $currentRow, $fob);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            $sheet->setCellValue($column . $currentRow, $totalPayable);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            $sheet->setCellValue($column . $currentRow, $profit);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            // Apply percentage formatting to Profit%
            $sheet->setCellValue($column . $currentRow, $profitPercentage / 100); // Excel expects decimal for percentage
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_PERCENTAGE_00);

            $currentRow++;
        }

        $currentRow += 2;
    }

    // Clear output buffer before sending headers
    if (ob_get_level()) {
        ob_end_clean();
    }

    header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
    header('Content-Disposition: attachment; filename="FGL_FP_Sheet.xlsx"');
    $writer = new Xlsx($spreadsheet);
    $writer->save('php://output');
    exit;
} else {
    echo "No data available for download.";
}
?>
