<?php
session_start();
require 'vendor/autoload.php';

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Style\NumberFormat;



if (!empty($_SESSION['tableData'])) {
    $spreadsheet = new Spreadsheet();
    $sheet = $spreadsheet->getActiveSheet();

    $currentRow = 1;

    foreach ($_SESSION['tableData'] as $table) {
        $title = "Payment from direct receipts - {$table['factory']}";
        $sheet->setCellValue('A' . $currentRow, $title);
        $sheet->getStyle('A' . $currentRow)->getFont()->setBold(true);
        $currentRow += 2;

        $headers = ['Buyer', 'Buyer Name', 'Final Invoice', 'AX Invoice','Factory Name', 'Date', 'Sales Value in US$','Purchase Value in US$', 'Profit', 'Profit%','AP Aging Availability'];
        $column = 'A';
        foreach ($headers as $header) {
            $sheet->setCellValue($column++ . $currentRow, $header);
        }
        $sheet->getStyle('A' . $currentRow . ':' . $column . $currentRow)->getFont()->setBold(true);
        $currentRow++;

        foreach ($table['data'] as $rowData) {
            $column = 'A';
        
            // Calculate FOB and PUR_VALUE from rowData
            $fob = floatval(str_replace(',', '', $rowData['FOB'] ?? '0'));
            $pur_val = floatval(str_replace(',', '', $rowData['PUR_VALUE'] ?? '0'));
        
            // Calculate profit and profit percentage
            $profit = $fob - $pur_val;  // Assuming PUR_VALUE is the cost to subtract from the sales value (FOB)
            $profitPercentage = ($fob > 0) ? ($profit / $fob * 100) : 0;  // Ensure division by zero is handled
        
            // Write data to cells
            $sheet->setCellValue($column++ . $currentRow, $rowData['Buyer'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Buyer_Name'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['F_Invoice'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['AX_Invoice'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['FACTORY'] ?? '');
            $sheet->setCellValue($column++ . $currentRow, $rowData['Date'] ?? '');
            $sheet->setCellValue($column . $currentRow, $fob);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            $sheet->setCellValue($column . $currentRow, $pur_val);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            $sheet->setCellValue($column . $currentRow, $profit);
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_NUMBER_COMMA_SEPARATED1);
            $sheet->setCellValue($column . $currentRow, $profitPercentage / 100); // Excel expects decimal for percentage
            $sheet->getStyle($column++ . $currentRow)->getNumberFormat()->setFormatCode(NumberFormat::FORMAT_PERCENTAGE_00);
            
            $sheet->setCellValue($column . $currentRow, $rowData['Availability'] ?? '');
        
            $currentRow++;
        }


        $currentRow += 2;
    }

    // Clear output buffer before sending headers
    if (ob_get_level()) {
        ob_end_clean();
    }

    header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
    header('Content-Disposition: attachment; filename="BomDataDownload.xlsx"');
    header('Cache-Control: max-age=0');

    $writer = new Xlsx($spreadsheet);
    $writer->save('php://output');
    exit;
} else {
    echo "No data available for download.";
}


?>
